/*
 * Copyright (C) 2022 Jonathan Certes
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */

#define TRACE_WITH_VERILATOR

#include "systemc.h"
#include "Vsynthesis.h"

#ifdef TRACE_WITH_VERILATOR
  #include "verilated_vcd_sc.h"
#endif

/******************************************************************************/

/**
 * \brief
 *  Generates input signals for the testbench.
 */
SC_MODULE( stimulus ) {
  sc_out <bool>     btn0;
  sc_out <uint32_t> sw;

  void stimulusGen( void ) {
    btn0.write( true );
    sw.write(    0x0 );

    wait(  45, SC_NS );
    btn0.write( false );

    wait(  40, SC_NS );
    sw.write(    0xd );
    wait(  40, SC_NS );
    btn0.write( true );
    wait(  40, SC_NS );
    btn0.write( false );

    wait( 100, SC_NS );
    sc_stop();
  }

  SC_CTOR(stimulus) {
    SC_THREAD(stimulusGen);
  }
};

/******************************************************************************/

int sc_main(
  int argc,
  char* argv[]
) {
  sc_signal <bool>     btn0;
  sc_signal <uint32_t> sw;
  sc_signal <uint32_t> led;

  sc_clock clk( "i_clkgen",
                20, SC_NS,      // period
                0.5,            // duty cycle
                10, SC_NS, true // starting edge
              );

  stimulus i_stim("i_stim");
           i_stim.btn0( btn0 );
           i_stim.sw(   sw   );

  Vsynthesis i_dut("i_dut");
             i_dut.clk(  clk  );
             i_dut.btn0( btn0 );
             i_dut.sw(   sw   );
             i_dut.led(  led  );

  sc_core::sc_start(sc_core::SC_ZERO_TIME);

  /*
   * creation of a VCD file:
   */
  #ifdef TRACE_WITH_VERILATOR

    VerilatedVcdSc* theOutput = new VerilatedVcdSc;
    Verilated::traceEverOn(true);
    i_dut.trace(theOutput, 99); // depth in the hierarchy
    theOutput->open("output.vcd");

  #else // trace with systemC native functions:

    sc_trace_file* theOutput;
    theOutput = sc_create_vcd_trace_file( "output" );
    sc_trace( theOutput, btn0, "btn0" );
    sc_trace( theOutput,  clk,  "clk" );
    sc_trace( theOutput,   sw,   "sw", 4 );
    sc_trace( theOutput,  led,  "led", 4 );

  #endif

  sc_start();  // run forever

  /*
   * closing VCD file:
   */
  #ifdef TRACE_WITH_VERILATOR
    theOutput->close();
  #else
    sc_close_vcd_trace_file(theOutput);
  #endif

  return 0;
}

